/** 
 * Copyright (C) 2008 Alan Ross, ( http://lab.adjazent.com )
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
*/
package ar.particlepainter.paint
{
	import ar.particlepainter.particles.ShapeParticle;
	
	import flash.display.BitmapData;
	import flash.display.Shape;	

	/**
	 * @author Alan Ross
	 */
	public class ShapePaint implements IPaint
	{
		private const NUM_PARTICLES: int = 1000;
		
		private const PI2: Number = Math.PI * 2;
		
		private var _canvas: BitmapData;
		
		private var _radius: int;
		
		private var _particles: ShapeParticle;

		/**
		 * Create a new <code>ShapePaint</code> object.
		 */
		public function ShapePaint( canvas: BitmapData, radius: int = 20 )
		{
			_canvas = canvas;
			
			_radius = radius;
			
			var p: ShapeParticle;
			
			p = _particles = new ShapeParticle();
			
			var n: int = NUM_PARTICLES;
			
			var s: BitmapData = new BitmapData( 2, 2, true );
			var shape: Shape = new Shape( );
			shape.graphics.beginFill( 0x08ea8a );
			shape.graphics.drawCircle(2, 2, 2);
			shape.graphics.endFill();
			s.draw(shape);
			
			while( --n > -1 )
			{
				p = p.next = new ShapeParticle();
				p.shape = s;
			}
		}
		
		/**
		 * @inheritDoc
		 */
		public function paint( mouseX: int, mouseY: int, mouseDown: Boolean ): void
		{
			var temp: Number;
			
			var p: ShapeParticle = _particles.next;
			
			do
			{	
				--p.restLife;
				
				if( p.restLife < 0 )
				{
					if( mouseDown )
					{
						temp = Math.random() * PI2;
						
						p.distToOrigin = ( _radius * Math.random() );
						
						p.x = mouseX + ( Math.sin( temp ) * p.distToOrigin );
						p.y = mouseY + ( Math.cos( temp ) * p.distToOrigin );
						
						p.dirX = Math.sin( temp );
						p.dirY = Math.cos( temp );
					}
					else
					{
						p.x = -1;
						p.y = -1;
					}	
					
					p.restLife = p.lifespan;
				}
				else
				{
					if( p.x != -1 && p.y != -1 )
					{
						p.x += ( p.dirX * ( p.size ) );
						p.y += ( p.dirY * ( p.size ) );
						
						p.paint( _canvas );
					}
				}
				
				p = p.next;
			} 
			while( p );
		}
	}
}
